%% Visual Clustering of Parallel Coordinates
% @author Mazurek/Bartalsky
%% gui_main.m parameters
% * *Description:*
% * Gui of the parallel coordinates script. The script accepts .mat files
% * as input, these files should contain a matrix. The collums of this 
% * should be the dimensions of the parallel coordinates and the rows are
% * the observations.
% * *Input:*  
% * _hObject_ ... handle to figure
% * _eventdata_ ... reserved - to be defined in a future version of MATLAB
% * _handles_ ... structure with handles and user data (see GUIDATA)
% * _varargin_ ... command line arguments to myui (see VARARGIN)
function varargout = gui_main(varargin)
    gui_Singleton = 1;
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @gui_main_OpeningFcn, ...
                       'gui_OutputFcn',  @gui_main_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if nargin && ischar(varargin{1})
        gui_State.gui_Callback = str2func(varargin{1});
    end

    if nargout
        [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
    else
        gui_mainfcn(gui_State, varargin{:});
    end
end

%% OpeningFcn 
% * *Description:*
% Function that gets executed on start
function gui_main_OpeningFcn(hObject, eventdata, handles, varargin)
    handles.output = hObject;
    guidata(hObject, handles);

    %% Read Inputfile
    global UserSettings;
    UserSettings.dataset          = readInput('input/synthesized.mat');
    name=fieldnames(UserSettings.dataset);
    name=name{1};
    UserSettings.dataset = UserSettings.dataset.(name);
    % Normalize data
    for i = 1:size(UserSettings.dataset,2)
        UserSettings.dataset(:,i) = mat2gray(UserSettings.dataset(:,i));
    end
    
    %% define starting parameters
    UserSettings.subcoords= 4;
    UserSettings.angle      = 1;
    UserSettings.distance   = 1;
    UserSettings.alpha_c   = 0.99;
    
    %% set slider parameters
    set(handles.sliderAngle,    'Value', UserSettings.angle);
    UserSettings.numSteps = 11;
    set(handles.sliderAngle, 'Min', 1);
    set(handles.sliderAngle, 'Max', UserSettings.numSteps);
    set(handles.sliderAngle, 'SliderStep', [1/(UserSettings.numSteps-1) , 1/(UserSettings.numSteps-1) ]);
    set(handles.sliderDistance, 'Value', UserSettings.distance);
    set(handles.sliderDistance, 'Min', 1);
    set(handles.sliderDistance, 'Max', UserSettings.numSteps);
    set(handles.sliderDistance, 'SliderStep', [1/(UserSettings.numSteps-1) , 1/(UserSettings.numSteps-1) ]);
    set(handles.sliderStraight, 'Value', UserSettings.alpha_c);
    set(handles.sliderStraight, 'Min', 0);
    set(handles.sliderStraight, 'Max', 1);
    
    guidata(hObject, handles);
    
    updateOutput(hObject, handles);
end

%% updateOutput
% * *Description:*
% * function that updates the figures
function updateOutput(hObject, handles)
    %% update parameters
    global UserSettings;    
    UserSettings.angle      =       get(handles.sliderAngle,      'Value');    
    UserSettings.distance   =       get(handles.sliderDistance,   'Value');    
    UserSettings.alpha_c   =       get(handles.sliderStraight,   'Value');
    
    %% compute new clustering for parallel coordinates
    try
    [subM, valM]=main(UserSettings.dataset, UserSettings.subcoords, UserSettings.angle, UserSettings.distance, UserSettings.alpha_c);
    catch err
        fprintf(2,[getReport(err) '\n']);
    end
    
    %% draw original parallel coordinates
    axes(handles.axesColor);
        cla;
        plotSubs(UserSettings.dataset, subM);
    
    %% draw clustered parallel coordinates
    axes(handles.axesResult);
        cla;
        plotBezier(UserSettings.dataset, valM);
    
    %% set texts in figure to match computations
    set(handles.txtAngle,   'String',           num2str(UserSettings.angle));
    set(handles.txtDistance,    'String',           num2str(UserSettings.distance));
    set(handles.txtStraight,    'String',           num2str(UserSettings.alpha_c));
    set(handles.sliderDistance, 'Min', 1);
    set(handles.sliderDistance, 'Max', UserSettings.numSteps);
    set(handles.sliderDistance, 'SliderStep', [1/(UserSettings.numSteps-1) , 1/(UserSettings.numSteps-1) ]);
    set(handles.sliderAngle, 'Min', 1);
    set(handles.sliderAngle, 'Max', UserSettings.numSteps);
    set(handles.sliderAngle, 'SliderStep', [1/(UserSettings.numSteps-1) , 1/(UserSettings.numSteps-1) ]);
    
    guidata(hObject, handles);
end

function sliderAngle_Callback(hObject, eventdata, handles) %#ok<DEFNU>
    val=round(hObject.Value);
    hObject.Value=val;
    updateOutput(hObject, handles);
end

function sliderDistance_Callback(hObject, eventdata, handles) %#ok<DEFNU>
    global UserSettings;
    prev=UserSettings.distance;
    val=round(hObject.Value);
    if (mod(val,2)==0)
    if(val>prev)
    hObject.Value=val+1;
    else
    hObject.Value=val-1;   
    end
    else    
    hObject.Value=val;   
    end
    updateOutput(hObject, handles);
end

function sliderStraight_Callback(hObject, eventdata, handles) %#ok<DEFNU>
    updateOutput(hObject, handles);
end

%% updateOutput
% * *Description:*
% * Callback of the Dataset Button. The script accepts .mat files
% * as input, these files should contain a matrix. The collums of this 
% * should be the dimensions of the parallel coordinates and the rows are
% * the observations.
function btnDataset_Callback(hObject, eventdata, handles) %#ok<DEFNU>
    global UserSettings;
    cpy=UserSettings.dataset;
    %% Read Inputfile
    try
    data = uigetfile({'*.mat'},'Dataset auswhlen');
    UserSettings.dataset=load(['input/' char(data)]);
    name=fieldnames(UserSettings.dataset);
    name=name{1};
    UserSettings.dataset = UserSettings.dataset.(name);
    % Normalize data
    for i = 1:size(UserSettings.dataset,2)
        UserSettings.dataset(:,i) = mat2gray(UserSettings.dataset(:,i));
    end
    catch err
        UserSettings.dataset=cpy;
        fprintf(2,[getReport(err) '\n']);
    end
    %% define starting parameters for new dataset
    UserSettings.angle=1;
    UserSettings.distance=1;
    UserSettings.alpha_c=0.75;
    
    %set sliders
    set(handles.sliderAngle,    'Value', UserSettings.angle);
    set(handles.sliderDistance, 'Value', UserSettings.distance);
    set(handles.sliderStraight, 'Value', UserSettings.alpha_c);
    
    updateOutput(hObject, handles);
end

function varargout = gui_main_OutputFcn(hObject, eventdata, handles) 
    varargout{1} = handles.output;
end
